import { compose } from '../../helpers';
import { getComponentName } from '../../helpers/componentHelpers';
import {
  createStyleMixin,
  draggableMixin,
  createProxy,
  componentNameValidationMixin,
} from '../../mixins';
import { createDynamicDataMixin } from '../../mixins/createDynamicDataMixin';
import { TextClass } from '@descope-ui/descope-text/class';

export const componentName = getComponentName('apps-list');

const limitAbbreviation = (str, limit = 2) =>
  str
    .trim()
    .split(' ')
    .splice(0, limit)
    .map((s) => s[0]?.toUpperCase())
    .join('');

const itemRenderer = ({ name, icon, url }, _, ref) => `
  <a ${url ? `href="${url}" title="${url}"` : ''} target="_blank">
    <descope-list-item>
    <descope-avatar
      ${icon ? `img="${icon}"` : ''}
      ${name ? `display-name="${name}" abbr=${limitAbbreviation(name)}` : ''}
      size=${ref.size}
    ></descope-avatar>
    <descope-text
      variant="body1"
      mode="primary"
    >${name}</descope-text>
    </descope-list-item>
  </a>
`;

const customMixin = (superclass) =>
  class AppsListMixinClass extends superclass {
    get size() {
      return this.getAttribute('size') || 'sm';
    }
  };

export const AppsListClass = compose(
  createStyleMixin({
    mappings: {
      maxHeight: { selector: () => ':host' },
      minHeight: { selector: () => ':host' },
      hostDirection: { selector: () => ':host', property: 'direction' },
      itemsFontWeight: {
        selector: TextClass.componentName,
        property: TextClass.cssVarList.fontWeight,
      },
      itemsFontSize: {
        selector: TextClass.componentName,
        property: TextClass.cssVarList.fontSize,
      },
      itemsTextAlign: {
        selector: TextClass.componentName,
        property: TextClass.cssVarList.textAlign,
      },
    },
  }),
  createDynamicDataMixin({ itemRenderer, rerenderAttrsList: ['size'] }),
  draggableMixin,
  componentNameValidationMixin,
  customMixin
)(
  createProxy({
    slots: ['empty-state'],
    wrappedEleName: 'descope-list',
    excludeAttrsSync: ['tabindex', 'class', 'empty'],
    componentName,
    style: () => `
      :host {
        width: 100%;
        display: inline-flex;
      }

      descope-text::part(text-wrapper) {
        display: -webkit-box;
        -webkit-line-clamp: 2;
        -webkit-box-orient: vertical;
        overflow: hidden;
      }

      a {
        text-decoration: none;
      }

      descope-text {
        ${TextClass.cssVarList.hostDirection}: var(${AppsListClass.cssVarList.hostDirection});
      }
    `,
  })
);
