import { formats } from './formats';

export const isValidTimestamp = (val) => !Number.isNaN(Number(val));

export const isNumber = (val) => !!String(val || '').trim() && !Number.isNaN(Number(val));

export const getTimestampParts = (timestamp) => {
  const date = newDate(timestamp);
  const year = date.getFullYear();
  const month = date.getMonth() + 1;
  const day = date.getDate();
  return [year, month, day];
};

export const formatTimestamp = (timestamp, format) => {
  const [year, month, day] = getTimestampParts(timestamp);

  const parts = {
    DD: String(day).padStart(2, '0'),
    MM: String(month).padStart(2, '0'),
    YYYY: String(year),
  };

  return format.replace(/DD|MM|YYYY/g, (match) => parts[match]);
};

// polyfill for safari Date API (which doesn't accept "YYYY-MM-DD" string as value)
export const newDate = (date) => {
  if (typeof date === 'number') {
    return new Date(date);
  }
  if (typeof date === 'string') {
    return new Date(date.replace(/-/g, '/'));
  }
  return new Date();
};

export const getCurrentTime = () => newDate().getTime();
export const getFullYear = (timestamp) => newDate(timestamp).getFullYear().toString();
export const getMonth = (timestamp) => (newDate(timestamp).getMonth() + 1).toString();
export const getCurrentDay = () => newDate().getDate();
export const getCurrentYear = () => newDate().getFullYear().toString();

// Vaadin uses "constructed stylesheet" to hide the host in dialog components.
// To override it, we need to push an overriding constructed stylesheet to the shadow DOM
// opened issue in vaadin: https://github.com/vaadin/web-components/issues/7979
export const overrideConstructedStylesheet = (ele) => {
  const cs = new CSSStyleSheet();
  cs.insertRule(':host{display:block!important;}');
  ele?.shadowRoot?.adoptedStyleSheets?.push(cs);
};

export const parseDateString = (val, format) => {
  const trimmed = val.trim?.();
  if (!trimmed) return null;
  return formats[format].getDate(trimmed);
};
