import { createStyleMixin, draggableMixin, componentNameValidationMixin } from '../../mixins';
import { createBaseClass } from '../../baseClasses/createBaseClass';
import { compose } from '../../helpers';
import { getComponentName } from '../../helpers/componentHelpers';
import { activeableMixin } from '../../mixins/activableMixin';
import { injectStyle } from '@descope-ui/common/components-helpers';

export const componentName = getComponentName('list-item');

const customMixin = (superclass) =>
  class ListItemMixinClass extends superclass {
    constructor() {
      super();

      this.attachShadow({ mode: 'open' }).innerHTML = `
      <slot></slot>
    `;

      injectStyle(
        `
      slot {
          width: 100%;
          display: flex;
          overflow: hidden;
          box-sizing: border-box;
      }
      :host {
        display: block;
      }
        `,
        this
      );
    }
  };

export const ListItemClass = compose(
  createStyleMixin({
    mappings: {
      padding: {},
      backgroundColor: {},
      borderColor: {},
      borderStyle: {},
      borderWidth: {},
      borderRadius: {},
      outline: {},
      cursor: {},
      gap: {},
      maxWidth: { selector: () => ':host' },
      alignItems: {},
      flexDirection: {},
      transition: {},
    },
  }),
  draggableMixin,
  componentNameValidationMixin,
  customMixin,
  activeableMixin
)(createBaseClass({ componentName, baseSelector: 'slot' }));
