import globals from '../../globals';
import { getThemeRefs, createHelperVars, useVar } from '../../../helpers/themeHelpers';
import { ListClass, componentName } from '../../../components/descope-list/ListClass';

const globalRefs = getThemeRefs(globals);

const compVars = ListClass.cssVarList;

const [helperTheme, helperRefs, helperVars] = createHelperVars(
  { shadowColor: '#00000020' },
  componentName
);

const { shadowColor } = helperRefs;

const list = {
  ...helperTheme,

  [compVars.hostWidth]: '100%',
  [compVars.backgroundColor]: globalRefs.colors.surface.main,
  [compVars.fontFamily]: globalRefs.fonts.font1.family,
  [compVars.borderColor]: globalRefs.colors.surface.light,
  [compVars.borderStyle]: 'solid',
  [compVars.borderWidth]: globalRefs.border.xs,
  [compVars.borderRadius]: globalRefs.radius.sm,
  [compVars.gap]: globalRefs.spacing.md,
  [compVars.verticalPadding]: globalRefs.spacing.lg,
  [compVars.horizontalPadding]: globalRefs.spacing.lg,
  [compVars.boxShadow]: `${globalRefs.shadow.wide.sm} ${shadowColor}, ${globalRefs.shadow.narrow.sm} ${shadowColor}`,
  [compVars.maxHeight]: '100%',
  [compVars.hostDirection]: globalRefs.direction,
  [compVars.minItemsWidth]: '150px',

  _empty: {
    [compVars.minHeight]: '150px',
    [compVars.emptyStateTextColor]: globalRefs.colors.surface.dark,
    [compVars.emptyStateTextFontFamily]: globalRefs.fonts.font1.family,
  },

  variant: {
    tiles: {
      [compVars.gridTemplateColumns]: `repeat(auto-fit, minmax(min(${useVar(
        compVars.minItemsWidth
      )}, 100%), 1fr))`,
      [compVars.maxItemsWidth]: '200px',
      [compVars.itemsHorizontalAlign]: 'center',
    },
  },
};

export default list;

export const vars = {
  ...compVars,
  ...helperVars,
};
